-- VHDL Entity Board.FPGA_sensorsInterface.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:35 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.numeric_std.all;

ENTITY FPGA_sensorsInterface IS
   GENERIC( 
      ioBitNb   : positive := 20;
      jtagBitNb : positive := 3
   );
   PORT( 
      clock   : IN     std_ulogic;
      reset_n : IN     std_uLogic;
      sCl     : IN     std_logic;
      jtagIo  : OUT    std_ulogic_vector (1 TO jtagBitNb);
      I_O     : INOUT  std_logic_vector (1 TO ioBitNb);
      sDa     : INOUT  std_logic
   );

-- Declarations

END FPGA_sensorsInterface ;





-- VHDL Entity Sensors.sensorsInterface.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:35 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY sensorsInterface IS
   GENERIC( 
      i2cBitNb                : positive := 10;
      kartBaseAddress         : positive := 16#51#;
      ledsBaseAddress         : natural  := 12;
      ledNb                   : positive := 4;
      testOutBitNb            : positive := 8;
      endSwitchBaseAddress    : natural  := 8;
      endSwitchNb             : positive := 1;
      hallBaseAddress         : natural  := 0;
      hallSensorNb            : positive := 2;
      hallCountBitNb          : positive := 16;
      proximityBaudRateDivide : positive := 100;
      proximityBaseAddress    : natural  := 12;
      proximitySensorNb       : positive := 2;
      ambientlightBitNb       : positive := 16;
      proximityBitNb          : positive := 16;
      rangeBaseAddress        : natural  := 16;
      rangeBitNb              : positive := 16;
      rangeCounterBitNb       : positive := 16
   );
   PORT( 
      clock         : IN     std_ulogic;
      distancePulse : IN     std_ulogic;
      endSwitches   : IN     std_ulogic_vector (1 TO endSwitchNb);
      hallPulses    : IN     std_ulogic_vector (1 TO hallSensorNb);
      proxySClIn    : IN     std_ulogic_vector (1 TO proximitySensorNb);
      proxySDaIn    : IN     std_ulogic_vector (1 TO proximitySensorNb);
      reset         : IN     std_ulogic;
      sClIn         : IN     std_uLogic;
      sDaIn         : IN     std_uLogic;
      distanceStart : OUT    std_ulogic;
      leds          : OUT    std_ulogic_vector (1 TO ledNb);
      proxySClOut   : OUT    std_ulogic_vector (1 TO proximitySensorNb);
      proxySDaOut   : OUT    std_ulogic_vector (1 TO proximitySensorNb);
      sDaOut        : OUT    std_uLogic;
      testOut       : OUT    std_ulogic_vector (1 TO testOutBitNb)
   );

-- Declarations

END sensorsInterface ;





-- VHDL Entity Sensors.hallCounters.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:35 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY hallCounters IS
   GENERIC( 
      sensorNb   : positive := 2;
      countBitNb : positive := 16
   );
   PORT( 
      clock       : IN     std_ulogic;
      hallPulses  : IN     std_ulogic_vector (1 TO sensorNb);
      reset       : IN     std_ulogic;
      zeroPos     : IN     std_ulogic;
      hallChanged : OUT    std_ulogic_vector (1 TO sensorNb);
      position    : OUT    unsigned (sensorNb*countBitNb-1 DOWNTO 0)
   );

-- Declarations

END hallCounters ;





ARCHITECTURE RTL OF hallCounters IS

  signal pulsesDelayed, pulsesChanged: std_ulogic_vector(1 to sensorNb);
  type counterSetType is array(1 to sensorNb) of unsigned(countBitNb-1 downto 0);
  signal counterSet: counterSetType;

BEGIN
  ------------------------------------------------------------------------------
                                                             -- find transitions
  delayPulses: process(reset, clock)
  begin
    if reset = '1' then
      pulsesDelayed <= (others => '0');
    elsif rising_edge(clock) then
      pulsesDelayed <= hallPulses;
    end if;
  end process delayPulses;

  pulsesChanged <= hallPulses xor pulsesDelayed;
  hallChanged <= pulsesChanged;

  ------------------------------------------------------------------------------
                                                      -- count transition number
  countPulses: process(reset, clock)
  begin
    if reset = '1' then
      counterSet <= (others => (others => '0'));
    elsif rising_edge(clock) then
      for index in pulsesChanged'range loop
        if pulsesChanged(index) = '1' then
          counterSet(index) <= counterSet(index) + 1;
        end if;
      end loop;
    end if;
  end process countPulses;

  ------------------------------------------------------------------------------
                                                        -- pack to single vector
  assignWords: process(counterSet)
    variable offset : natural;
  begin
    for index in counterSet'range loop
      offset := (index-1) * counterSet(index)'length;
      position(counterSet(index)'high+offset downto offset) <= counterSet(index);
    end loop;
  end process assignWords;

END ARCHITECTURE RTL;




-- VHDL Entity I2C.i2cTransceiver.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 11:18:30 19.05.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY i2cTransceiver IS
   GENERIC( 
      dataBitNb : positive := 10
   );
   PORT( 
      sDaIn        : IN     std_ulogic;
      clock        : IN     std_ulogic;
      reset        : IN     std_ulogic;
      dataIn       : OUT    std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      dataValid    : OUT    std_ulogic;
      sCl          : IN     std_ulogic;
      registerAddr : OUT    unsigned (dataBitNb-2-1 DOWNTO 0);
      writeData    : OUT    std_ulogic;
      sDaOut       : OUT    std_ulogic;
      dataOut      : IN     std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      chipAddr     : OUT    unsigned (dataBitNb-3-1 DOWNTO 0);
      isSelected   : IN     std_ulogic
   );

-- Declarations

END i2cTransceiver ;





PACKAGE CommonLib IS

  function requiredBitNb (val : integer) return integer;

END CommonLib;




PACKAGE BODY CommonLib IS

  function requiredBitNb (val : integer) return integer is
    variable powerOfTwo, bitNb : integer;
  begin
    powerOfTwo := 1;
    bitNb := 0;
    while powerOfTwo <= val loop
      powerOfTwo := 2 * powerOfTwo;
      bitNb := bitNb + 1;
    end loop;
    return bitNb;
  end requiredBitNb;

END CommonLib;




-- library Common;
  use work.CommonLib.all;

architecture RTL of i2cTransceiver is
                                                                     -- receiver
  signal clDelayed      : std_uLogic;
  signal daDelayed      : std_uLogic;
  signal clRising       : std_uLogic;
  signal clFalling      : std_uLogic;
  signal daRising       : std_uLogic;
  signal daFalling      : std_uLogic;
  signal startCondition : std_uLogic;
  signal stopCondition  : std_uLogic;
  signal bitCounter     : unsigned(requiredBitNb(dataBitNb)-1 downto 0);
  signal inputShiftReg  : std_ulogic_vector(dataBitNb-2 downto 0);
  signal clRisingDelayed: std_uLogic;
  signal endOfWord      : std_uLogic;
  signal i2cByteValid   : std_uLogic;
  signal i2cByte        : std_ulogic_vector(dataIn'range);
                                                                          -- ack
  signal ackMoment      : std_uLogic;
  signal sendAck        : std_uLogic;
                                                                      -- decoder
  type   rxStateType    is (idle, readChipAddress, readRegAddress,
                            incAddress1, incAddress);
  signal rxState        : rxStateType;
  signal chipAddress    : unsigned(chipAddr'range);
  signal registerAddress: unsigned(registerAddr'range);
  signal writeMode      : std_uLogic;

  signal loadOutShiftReg: std_uLogic;
  signal enSdaOut       : std_uLogic;
  signal outputShiftReg : std_ulogic_vector(dataBitNb-2-1 downto 0);

begin
  --============================================================================
                                             -- start, stop and other conditions
  delayInputs: process(reset, clock)
  begin
    if reset = '1' then
      clDelayed <= '1';
      daDelayed <= '1';
      clRisingDelayed <= '0';
    elsif rising_edge(clock) then
      clDelayed <= sCl;
      daDelayed <= sDaIn;
      clRisingDelayed <= clRising;
    end if;
  end process delayInputs;

  clRising <= '1' when (sCl = '1') and (clDelayed = '0') else '0';
  clFalling <= '1' when (sCl = '0') and (clDelayed = '1') else '0';
  daRising <= '1' when (sDaIn = '1') and (daDelayed = '0') else '0';
  daFalling <= '1' when (sDaIn = '0') and (daDelayed = '1') else '0';
  startCondition <= '1' when (daFalling = '1') and (sCl = '1') else '0';
  stopCondition <= '1' when (daRising = '1') and (sCl = '1') else '0';

  ------------------------------------------------------------------------------
                                                                  -- bit counter
  countBitNb: process(reset, clock)
  begin
    if reset = '1' then
      bitCounter <= (others => '0');
    elsif rising_edge(clock) then
      if startCondition = '1' then
        bitCounter <= (others => '0');
      elsif stopCondition = '1' then
        bitCounter <= (others => '0');
      elsif clFalling = '1' then
        if bitCounter < dataBitNb-1 then
          bitCounter <= bitCounter + 1;
        else
          bitCounter <= to_unsigned(1, bitCounter'length);
        end if;
      end if;
    end if;
  end process countBitNb;

  endOfWord <= '1' when (clRisingDelayed = '1') and (bitCounter = dataBitNb-1)
    else '0';

  --============================================================================
                                                         -- input shift register
  shiftInputBits: process(reset, clock)
  begin
    if reset = '1' then
      inputShiftReg <= (others => '0');
    elsif rising_edge(clock) then
      if clRising = '1' then
        inputShiftReg(inputShiftReg'high downto 1) <= inputShiftReg(inputShiftReg'high-1 downto 0);
        inputShiftReg(0) <= sDaIn;
      end if;
    end if;
  end process shiftInputBits;

  ------------------------------------------------------------------------------
                                                -- read byte from shift register
  i2cByteValid <= '1' when (bitCounter = i2cByte'length) and (clFalling = '1')
    else '0';
  i2cByte <= inputShiftReg(i2cByte'range);

  storeByte: process(reset, clock)
  begin
    if reset = '1' then
      dataIn <= (others => '0');
    elsif rising_edge(clock) then
      if i2cByteValid = '1' then
        dataIn <= i2cByte;
      end if;
    end if;
  end process storeByte;

  --============================================================================
                                                           -- address extraction
  rxFsm: process(reset, clock)
  begin
    if reset = '1' then
      rxState <= idle;
    elsif rising_edge(clock) then
      if stopCondition = '1' then
        rxState <= idle;
      elsif startCondition = '1' then
        rxState <= readChipAddress;
      elsif endOfWord = '1' then
        case rxState is
          when readChipAddress =>
            if writeMode = '1' then
              rxState <= readRegAddress;
            else
              rxState <= incAddress1;
            end if;
          when readRegAddress =>
            rxState <= incAddress1;
          when incAddress1 =>
            rxState <= incAddress;
          when incAddress =>
            if (writeMode = '0') and (sDaIn = '1') then
              rxState <= idle;
            end if;
          when others => null;
        end case;
      end if;
    end if;
  end process rxFsm;

  dataValid <= endOfWord when (rxState = incAddress1) or (rxState = incAddress)
    else '0';

  ------------------------------------------------------------------------------
                                                  -- provide running i2c address
  updateAddress: process(reset, clock)
  begin
    if reset = '1' then
      chipAddress <= (others => '0');
      registerAddress <= (others => '0');
      writeMode <= '0';
    elsif rising_edge(clock) then
      if i2cByteValid = '1' then
        case rxState is
          when readChipAddress =>
            chipAddress <= resize(
              shift_right(unsigned(i2cByte), 1),
              chipAddress'length
            );
            if i2cByte(0) = '0' then -- i2cByte(0)=0 -> master write to slave
              writeMode <= '1';
            else
              writeMode <= '0';
              registerAddress <= registerAddress + 1;
            end if;
          when readRegAddress =>
            registerAddress <= resize(unsigned(i2cByte)-1, registerAddress'length);
          when incAddress1 | incAddress =>
            registerAddress <= registerAddress + 1;
          when others => null;
        end case;
      end if;
    end if;
  end process updateAddress;

  chipAddr <= chipAddress;
  registerAddr <= registerAddress;
  writeData <= writeMode;

  ------------------------------------------------------------------------------
                                                                  -- acknowledge
  findAckMoment: process(reset, clock)
  begin
    if reset = '1' then
      ackMoment <= '0';
      sendAck <= '0';
    elsif rising_edge(clock) then
      if (bitCounter = dataBitNb-2) and (clFalling = '1') then
        ackMoment <= '1';
      elsif clFalling = '1' then
        ackMoment <= '0';
      end if;
      if ackMoment = '0' then
        sendAck <= '0';
      elsif isSelected = '1' then
        if writeMode = '1' then
          sendAck <= '1';
        elsif rxState = readChipAddress then
          sendAck <= '1';
        end if;
      end if;
    end if;
  end process findAckMoment;

  --============================================================================
                                                              -- output controls
  controlAnswering: process(reset, clock)
  begin
    if reset = '1' then
      loadOutShiftReg <= '0';
      enSdaOut <= '0';
    elsif rising_edge(clock) then
      if endOfWord = '1' then
        loadOutShiftReg <= '1';
      elsif clRising = '1' then
        loadOutShiftReg <= '0';
      end if;
      if
        (bitCounter = 1) and
        ( (rxState = incAddress1) or (rxState = incAddress) ) and
        (writeMode = '0')
      then
        if sDaIn /= '0' then  -- let master finish ack pulse
          enSdaOut <= '1';
        end if;
      elsif ( (bitCounter = dataBitNb-2) and (clFalling = '1') ) or (rxState = idle) then
        enSdaOut <= '0';
      end if;
    end if;
  end process controlAnswering;

  ------------------------------------------------------------------------------
                                                        -- output shift register
  shiftOutputBits: process(reset, clock)
  begin
    if reset = '1' then
      outputShiftReg <= (others => '0');
    elsif rising_edge(clock) then
      if loadOutShiftReg = '1' then
        outputShiftReg <= (others => '1');
        outputShiftReg(dataOut'range) <= dataOut;
      elsif clFalling = '1' then
        outputShiftReg(outputShiftReg'high downto 1) <= outputShiftReg(outputShiftReg'high-1 downto 0);
        outputShiftReg(0) <= '0';
      end if;
    end if;
  end process shiftOutputBits;

  ------------------------------------------------------------------------------
                                                        -- reply to read request
  sDaOut <= '0' when sendAck = '1'
    else outputShiftReg(outputShiftReg'high) when enSdaOut = '1'
    else '1';

end RTL;




-- VHDL Entity Sensors.proximitySensorInterface.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:35 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY proximitySensorInterface IS
   GENERIC( 
      dataBitNb         : positive := 10;
      baudRateDivide    : positive := 660;
      sensorNb          : positive := 4;
      ambientLightBitNb : positive := 16;
      proximityBitNb    : positive := 16
   );
   PORT( 
      clock        : IN     std_ulogic;
      reset        : IN     std_ulogic;
      ambientLight : OUT    unsigned (sensorNb*ambientLightBitNb-1 DOWNTO 0);
      sDaOut       : OUT    std_ulogic_vector (1 TO sensorNb);
      sClOut       : OUT    std_ulogic_vector (1 TO sensorNb);
      sClIn        : IN     std_ulogic_vector (1 TO sensorNb);
      sDaIn        : IN     std_ulogic_vector (1 TO sensorNb);
      start        : IN     std_ulogic;
      proximity    : OUT    unsigned (sensorNb*proximityBitNB-1 DOWNTO 0)
   );

-- Declarations

END proximitySensorInterface ;





-- VHDL Entity Sensors.proximitySensorSequencer.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:35 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY proximitySensorSequencer IS
   GENERIC( 
      dataBitNb         : positive := 8;
      sensorNb          : positive := 2;
      ambientLightBitNb : positive := 16;
      proximityBitNb    : positive := 16
   );
   PORT( 
      clock        : IN     std_ulogic;
      reset        : IN     std_ulogic;
      ambientLight : OUT    unsigned (sensorNb*ambientLightBitNb-1 DOWNTO 0);
      proximity    : OUT    unsigned (sensorNb*proximityBitNb-1 DOWNTO 0);
      start        : IN     std_ulogic;
      txData       : OUT    std_ulogic_vector (dataBitNb-1 DOWNTO 0);
      txBusy       : IN     std_ulogic;
      txSend       : OUT    std_ulogic;
      rxDataValid  : IN     std_ulogic;
      rxData       : IN     std_ulogic_vector (sensorNb*dataBitNb-1 DOWNTO 0)
   );

-- Declarations

END proximitySensorSequencer ;





-- library Common;
  use work.CommonLib.all;

ARCHITECTURE RTL OF proximitySensorSequencer IS

  constant sequenceLength : positive := 6+4 + 2*6 + 5;
  signal sequenceCounter: unsigned(requiredBitNb(sequenceLength)-1 downto 0);

  constant i2cWordBitNb: positive := dataBitNb-2;
  subtype i2cWordType is std_ulogic_vector(i2cWordBitNb-1 downto 0);
  constant i2cStart                     : i2cWordType := X"00";
  constant i2cStop                      : i2cWordType := X"FF";
  constant i2cRead                      : i2cWordType := X"FF";
  constant distanceSensorAddress        : i2cWordType := X"26";
  constant commandRegisterAddress       : i2cWordType := X"80";
  constant ledCurrentRegisterAddress    : i2cWordType := X"83";
  constant lightRegisterAddress         : i2cWordType := X"85";
  constant proximityRegisterAddress     : i2cWordType := X"87";
  constant frequencyRegisterAddress     : i2cWordType := X"89";
  constant acquisitionStart             : i2cWordType := X"18";
  constant ledCurrentInit               : i2cWordType := X"14";
  constant acquisitionParamsInit        : i2cWordType := X"08";
  constant frequencyInit                : i2cWordType := X"03";
  constant timingsParamsInit            : i2cWordType := X"81";
  signal startStop: std_ulogic;
  signal i2cWord: i2cWordType;
  signal ack: std_ulogic;

  constant readLength : positive := 4;
  signal readCounter: unsigned(requiredBitNb(readLength)-1 downto 0);
  signal dataWord: unsigned(sensorNb*i2cWordBitNb-1 downto 0);
  signal ambientLightH, ambientLightL: unsigned(sensorNb*i2cWordBitNb-1 downto 0);
  signal proximityH, proximityL: unsigned(sensorNb*i2cWordBitNb-1 downto 0);

BEGIN
  --============================================================================
                                                          -- count send sequence
  countSequence: process(reset, clock)
  begin
    if reset = '1' then
      sequenceCounter <= (others => '0');
    elsif rising_edge(clock) then
      if sequenceCounter = 0 then
        if start = '1' then
          sequenceCounter <= sequenceCounter + 1;
        end if;
      else
        if txBusy = '0' then
          if sequenceCounter < sequenceLength then
            sequenceCounter <= sequenceCounter + 1;
          else
            sequenceCounter <= (others => '0');
          end if;
        end if;
      end if;
    end if;
  end process countSequence;

  --============================================================================
                                                            -- send I2C commands
  sendI2cCommands: process(sequenceCounter)
  begin
    startStop <= '0';
    i2cWord <= (others => '0');
    ack <= '1';
    case to_integer(sequenceCounter) is
                                             -- read ambient light and proximity
      when  1 => startStop <= '1'; i2cWord <= i2cStart;
      when  2 => i2cWord <= distanceSensorAddress;
      when  3 => i2cWord <= lightRegisterAddress;
      when  4 => startStop <= '1'; i2cWord <= i2cStart;
      when  5 => i2cWord <= distanceSensorAddress or X"01";
      when  6 => i2cWord <= i2cRead; ack <= '0';
      when  7 => i2cWord <= i2cRead; ack <= '0';
      when  8 => i2cWord <= i2cRead; ack <= '0';
      when  9 => i2cWord <= i2cRead;
      when 10 => startStop <= '1'; i2cWord <= i2cStop;
                                                 -- setup acquisition parameters
      when 11 => startStop <= '1'; i2cWord <= i2cStart;
      when 12 => i2cWord <= distanceSensorAddress;
      when 13 => i2cWord <= ledCurrentRegisterAddress;
      when 14 => i2cWord <= ledCurrentInit;
      when 15 => i2cWord <= acquisitionParamsInit;
      when 16 => startStop <= '1'; i2cWord <= i2cStop;
                                          -- setup frequency and time parameters
      when 17 => startStop <= '1'; i2cWord <= i2cStart;
      when 18 => i2cWord <= distanceSensorAddress;
      when 19 => i2cWord <= frequencyRegisterAddress;
      when 20 => i2cWord <= frequencyInit;
      when 21 => i2cWord <= timingsParamsInit;
      when 22 => startStop <= '1'; i2cWord <= i2cStop;
                                                      -- start a new acquisition
      when 23 => startStop <= '1'; i2cWord <= i2cStart;
      when 24 => i2cWord <= distanceSensorAddress;
      when 25 => i2cWord <= commandRegisterAddress;
      when 26 => i2cWord <= acquisitionStart;
      when 27 => startStop <= '1'; i2cWord <= i2cStop;
      when others => null;
    end case;
  end process sendI2cCommands;

  txData <= startStop & ack & i2cWord;
  txSend <= '1' when (sequenceCounter > 0) and (txBusy = '0')
    else '0';

  --============================================================================
                                                           -- read data from I2C
  ------------------------------------------------------------------------------
                                      -- read counter increments with input data
  readFSM: process(reset, clock)
  begin
    if reset = '1' then
      readCounter <= (others => '0');
    elsif rising_edge(clock) then
                                               -- start condition clears counter
      if (rxDataValid = '1') and (rxData(rxData'high) = '1') and (rxData(rxData'high) = '0') then
        readCounter <= (others => '0');
                                                 -- chip address starts counting
      elsif readCounter = 0 then
        if (rxDataValid = '1') and (rxData(dataBitNb-3 downto 0) = (distanceSensorAddress or X"01")) then
          readCounter <= readCounter + 1;
        end if;
      else
        if rxDataValid = '1' then
                                     -- acknowledge defines when to stop counter
          if rxData(dataBitNb-2) = '0' then
            readCounter <= readCounter + 1;
          else
            readCounter <= (others => '0');
          end if;
        end if;
      end if;
    end if;
  end process readFSM;

  ------------------------------------------------------------------------------
                                 -- remove start/stop and ACK bits from I2C data
  trimcontrolBits: process(rxData)
    variable regOffset, i2cOffset: natural;
  begin
    for index in 1 to sensorNb loop
      regOffset := i2cWordBitNb*(index-1);
      i2cOffset := dataBitNb*(index-1);
      dataWord(i2cWordBitNb-1+regOffset downto regOffset) <= unsigned(
        rxData(i2cWordBitNb-1+i2cOffset downto i2cOffset)
      );
    end loop;
  end process trimcontrolBits;

  ------------------------------------------------------------------------------
                                   -- store all high and then all low data bytes
  storeData: process(reset, clock)
  begin
    if reset = '1' then
      ambientLightH <= (others => '0');
      ambientLightL <= (others => '0');
      proximityH <= (others => '0');
      proximityL <= (others => '0');
    elsif rising_edge(clock) then
      if rxDataValid = '1' then
        case to_integer(readCounter) is
          when 1 => ambientLightH <= dataWord;
          when 2 => ambientLightL <= dataWord;
          when 3 => proximityH    <= dataWord;
          when 4 => proximityL    <= dataWord;
          when others => null;
        end case;
      end if;
    end if;
  end process storeData;

  ------------------------------------------------------------------------------
                                          -- put high and low bits back together
  interleaveBytes: process(
    ambientLightL, ambientLightH,
    proximityL, proximityH
  )
    variable regOffset, vectOffsetH, vectOffsetL: natural;
  begin
    for index in 1 to sensorNb loop
      regOffset := i2cWordBitNb*(index-1);
      vectOffsetL := 2*i2cWordBitNb*(index-1);
      vectOffsetH := vectOffsetL + i2cWordBitNb;
      ambientLight(i2cWordBitNb-1+vectOffsetL downto vectOffsetL) <=
        ambientLightL(i2cWordBitNb-1+regOffset downto regOffset);
      ambientLight(i2cWordBitNb-1+vectOffsetH downto vectOffsetH) <=
        ambientLightH(i2cWordBitNb-1+regOffset downto regOffset);
      proximity(i2cWordBitNb-1+vectOffsetL downto vectOffsetL) <=
        proximityL(i2cWordBitNb-1+regOffset downto regOffset);
      proximity(i2cWordBitNb-1+vectOffsetH downto vectOffsetH) <=
        proximityH(i2cWordBitNb-1+regOffset downto regOffset);
    end loop;
  end process interleaveBytes;

END ARCHITECTURE RTL;




-- VHDL Entity I2C.i2cTransmitter.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:23:30 19.05.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY i2cTransmitter IS
   GENERIC( 
      dataBitNb      : positive := 8;
      baudRateDivide : positive := 60
   );
   PORT( 
      sCl    : OUT    std_ulogic;
      clock  : IN     std_ulogic;
      reset  : IN     std_ulogic;
      dataIn : IN     std_ulogic_vector (dataBitNb-1 DOWNTO 0);
      send   : IN     std_ulogic;
      busy   : OUT    std_ulogic;
      sDa    : OUT    std_ulogic
   );

-- Declarations

END i2cTransmitter ;





-- library Common;
  use work.CommonLib.all;

architecture RTL of i2cTransmitter is

  signal sendStart: std_ulogic;
  signal sendStop: std_ulogic;
  signal dividerCounter: unsigned(requiredBitNb(baudRateDivide)-1 downto 0);
  signal dividerCounterReset: std_uLogic;
  signal send1: std_uLogic;
  signal send2: std_uLogic;
  signal sendSynch: std_uLogic;
  signal txCountEnable: std_uLogic;
  signal txShiftEnable: std_uLogic;
  signal phaseCounter: unsigned(1 downto 0);
  signal txShiftReg: std_ulogic_vector(dataIn'high downto 0);
  signal txSending: std_uLogic;
  signal txSendingDelayed: std_uLogic;
  signal txSendingEnd: std_uLogic;
  signal busyInt: std_uLogic;

begin
  ------------------------------------------------------------------------------
        -- check the MSB in order to differentiate start/stop from normal access
  checkMode: process(reset, clock)
  begin
    if reset = '1' then
      sendStart <= '0';
      sendStop <= '0';
    elsif rising_edge(clock) then
      if send = '1' then
        sendStart <= '0';
        sendStop <= '0';
        if dataIn(dataIn'high) = '1' then
          if dataIn(1) = '0' then
            sendStart <= '1';
          else
            sendStop <= '1';
          end if;
        end if;
      end if;
    end if;
  end process checkMode;

  ------------------------------------------------------------------------------
                                                           -- build I2c bit rate
  divide: process(reset, clock)
  begin
    if reset = '1' then
      dividerCounter <= (others => '0');
    elsif rising_edge(clock) then
      if dividerCounterReset = '1' then
        dividerCounter <= to_unsigned(1, dividerCounter'length);
      else
        dividerCounter <= dividerCounter + 1;
      end if;
    end if;
  end process divide;

  endOfCount: process(dividerCounter, send1)
  begin
    if dividerCounter = baudRateDivide then
      dividerCounterReset <= '1';
    else
      dividerCounterReset <= '0';
    end if;
  end process endOfCount;

  txCountEnable <= dividerCounterReset;

  shiftSend: process(reset, clock)
  begin
    if reset = '1' then
      send1 <= '0';
      send2 <= '0';
    elsif rising_edge(clock) then
      if send = '1' then
        send1 <= '1';
      elsif txCountEnable = '1' then
        send1 <= '0';
      end if;
      send2 <= send1;
    end if;
  end process shiftSend;

  sendSynch <= '1' when (send2 = '1') and (send1 = '0') else '0';

  buildPhases: process(reset, clock)
  begin
    if reset = '1' then
      phaseCounter <= (others => '0');
    elsif rising_edge(clock) then
      if sendSynch = '1' then
        phaseCounter <= (others => '0');
      elsif txCountEnable = '1' then
        phaseCounter <= phaseCounter + 1;
      end if;
    end if;
  end process buildPhases;

  txShiftEnable <= txCountEnable when (phaseCounter = 0) and (txSending = '1') else '0';

  ------------------------------------------------------------------------------
                                                              -- data serializer
  shiftReg: process(reset, clock)
  begin
    if reset = '1' then
      txShiftReg <= (others => '1');
    elsif rising_edge(clock) then
      if send = '1' then
        -- move ack bit from MSB to LSB
        txShiftReg <= dataIn(dataIn'high-2 downto 0) & dataIn(dataIn'high-1) & '0';
      elsif txShiftEnable = '1' then
        txShiftReg(txShiftReg'high downto 1) <= txShiftReg(txShiftReg'high-1 downto 0);
        -- fill shift register with 1's to determine when all has been sent
        txShiftReg(0) <= '1';
      end if;
    end if;
  end process shiftReg;

  ------------------------------------------------------------------------------
                                                                 -- serial clock
  buildClock: process(reset, clock)
  begin
    if reset = '1' then
      sCl <= '1';
    elsif rising_edge(clock) then
      if txSending = '1' then
        if phaseCounter = 1  then
          sCl <= '1';
        elsif phaseCounter = 3  then
          sCl <= '0';
        end if;
      end if;
    end if;
  end process buildClock;

  ------------------------------------------------------------------------------
                                                                  -- serial data
  buildData: process(reset, clock)
  begin
    if reset = '1' then
      sDa <= '1';
    elsif rising_edge(clock) then
      if txSending = '1' then
        if sendStart = '1' then
          if phaseCounter = 0 then
            sDa <= '1';
          elsif phaseCounter = 2 then
            sDa <= '0';
          end if;
        elsif sendStop = '1' then
          if phaseCounter = 0 then
            sDa <= '0';
          elsif phaseCounter = 2 then
            sDa <= '1';
          end if;
        elsif phaseCounter = 0 then
          sDa <= txShiftReg(txShiftReg'high);
        end if;
      end if;
    end if;
  end process buildData;

  ------------------------------------------------------------------------------
                                                                   -- byte frame
  frameEnvelope: process(reset, clock)
  begin
    if reset = '1' then
      txSending <= '0';
    elsif rising_edge(clock) then
      if sendSynch = '1' then
        txSending <= '1';
      elsif sendStart = '1' then
        if phaseCounter = 3 then
          txSending <= '0';
        end if;
      elsif sendStop = '1' then
        if phaseCounter = 2 then
          txSending <= '0';
        end if;
      elsif txShiftReg(txShiftReg'high-1 downto 0) = (txShiftReg'high-1 downto 0 => '1') then
        if phaseCounter = 3 then
          txSending <= '0';
        end if;
      end if;
    end if;
  end process frameEnvelope;

  delayEnvelope: process(reset, clock)
  begin
    if reset = '1' then
      txSendingDelayed <= '0';
    elsif rising_edge(clock) then
      txSendingDelayed <= txSending;
    end if;
  end process delayEnvelope;

  txSendingEnd <= '1' when (txSendingDelayed = '1') and (txSending = '0') else '0';

  buildBusy: process(reset, clock)
  begin
    if reset = '1' then
      busyInt <= '0';
    elsif rising_edge(clock) then
      if send = '1' then
        busyInt <= '1';
      elsif txSendingEnd = '1' then
        busyInt <= '0';
      end if;
    end if;
  end process buildBusy;

  busy <= busyInt;

end RTL;




-- VHDL Entity I2C.i2cReceiver.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 10:23:30 19.05.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY i2cReceiver IS
   GENERIC( 
      dataBitNb : positive := 8
   );
   PORT( 
      sDa       : IN     std_ulogic;
      clock     : IN     std_ulogic;
      reset     : IN     std_ulogic;
      dataOut   : OUT    std_ulogic_vector (dataBitNb-1 DOWNTO 0);
      dataValid : OUT    std_ulogic;
      sCl       : IN     std_ulogic
   );

-- Declarations

END i2cReceiver ;





-- library Common;
  use work.CommonLib.all;

architecture RTL of i2cReceiver is

  signal clDelayed: std_uLogic;
  signal daDelayed: std_uLogic;
  signal clRising: std_uLogic;
  signal daRising: std_uLogic;
  signal daFalling: std_uLogic;
  signal startCondition: std_uLogic;
  signal stopCondition: std_uLogic;
  signal dataShiftReg: std_ulogic_vector(dataBitNb-2 downto 0);
  signal bitCounter: unsigned(requiredBitNb(dataBitNb)-1 downto 0);
  signal clRisingDelayed: std_uLogic;
  signal endOfWord: std_uLogic;

begin
  ------------------------------------------------------------------------------
                                             -- start, stop and other conditions
  delayInputs: process(reset, clock)
  begin
    if reset = '1' then
      clDelayed <= '1';
      daDelayed <= '1';
      clRisingDelayed <= '0';
    elsif rising_edge(clock) then
      clDelayed <= sCl;
      daDelayed <= sDa;
      clRisingDelayed <= clRising;
    end if;
  end process delayInputs;

  clRising <= '1' when (sCl = '1') and (clDelayed = '0') else '0';
  daRising <= '1' when (sDa = '1') and (daDelayed = '0') else '0';
  daFalling <= '1' when (sDa = '0') and (daDelayed = '1') else '0';
  startCondition <= '1' when (daFalling = '1') and (sCl = '1') else '0';
  stopCondition <= '1' when (daRising = '1') and (sCl = '1') else '0';
  endOfWord <= '1' when (clRisingDelayed = '1') and (bitCounter = dataBitNb-1) else '0';

  ------------------------------------------------------------------------------
                                                          -- data shift register
  shiftReg: process(reset, clock)
  begin
    if reset = '1' then
      dataShiftReg <= (others => '0');
    elsif rising_edge(clock) then
      if clRising = '1' then
        dataShiftReg(dataShiftReg'high downto 1) <= dataShiftReg(dataShiftReg'high-1 downto 0);
        dataShiftReg(0) <= sDA;
      end if;
    end if;
  end process shiftReg;

  ------------------------------------------------------------------------------
                                                                  -- bit counter
  countBitNb: process(reset, clock)
  begin
    if reset = '1' then
      bitCounter <= (others => '0');
    elsif rising_edge(clock) then
      if startCondition = '1' then
        bitCounter <= (others => '0');
      elsif stopCondition = '1' then
        bitCounter <= (others => '0');
      elsif clRising = '1' then
        if bitCounter <= dataBitNb then
          bitCounter <= bitCounter + 1;
        else
          bitCounter <= (others => '0');
        end if;
      elsif endOfWord = '1' then
        bitCounter <= (others => '0');
      end if;
    end if;
  end process countBitNb;

  ------------------------------------------------------------------------------
               -- output data and control
               -- start and stop: MSB is '1', LSB specifies start/stop
               -- data words: MSB is '0', next is ack, 8 LSBs are data word
               -- could be made combinatorial, but sequential is easier to debug
  sendWord: process(reset, clock)
  begin
    if reset = '1' then
      dataValid <= '0';
      dataOut <= (others => '0');
    elsif rising_edge(clock) then
      dataValid <= '0';
      if startCondition = '1' then
        dataValid <= '1';
        dataOut <= (others => '0');
        dataOut(dataOut'high) <= '1';
      elsif stopCondition = '1' then
        dataValid <= '1';
        dataOut <= (others => '1');
        dataOut(dataOut'high) <= '1';
      elsif endOfWord = '1' then
        dataValid <= '1';
        dataOut <= '0' & dataShiftReg(0) & dataShiftReg(dataShiftReg'high downto 1);
      end if;
    end if;
  end process sendWord;

end RTL;




--
-- VHDL Architecture Sensors.proximitySensorInterface.struct
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:35 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

-- LIBRARY I2C;
-- LIBRARY Sensors;

ARCHITECTURE struct OF proximitySensorInterface IS

   -- Architecture declarations

   -- Internal signal declarations
   SIGNAL txData    : std_ulogic_vector(dataBitNb-1 DOWNTO 0);
   SIGNAL rxData    : std_ulogic_vector(sensorNb*dataBitNb-1 DOWNTO 0);
   SIGNAL txSend    : std_ulogic;
   SIGNAL txBusy    : std_ulogic;
   SIGNAL sDaOut1   : std_ulogic;
   SIGNAL dataValid : std_ulogic_vector(1 TO sensorNb);
   SIGNAL sClOut1   : std_ulogic;


   -- Component Declarations
   COMPONENT i2cReceiver
   GENERIC (
      dataBitNb : positive := 8
   );
   PORT (
      sDa       : IN     std_ulogic ;
      clock     : IN     std_ulogic ;
      reset     : IN     std_ulogic ;
      dataOut   : OUT    std_ulogic_vector (dataBitNb-1 DOWNTO 0);
      dataValid : OUT    std_ulogic ;
      sCl       : IN     std_ulogic 
   );
   END COMPONENT;
   COMPONENT i2cTransmitter
   GENERIC (
      dataBitNb      : positive := 8;
      baudRateDivide : positive := 60
   );
   PORT (
      sCl    : OUT    std_ulogic ;
      clock  : IN     std_ulogic ;
      reset  : IN     std_ulogic ;
      dataIn : IN     std_ulogic_vector (dataBitNb-1 DOWNTO 0);
      send   : IN     std_ulogic ;
      busy   : OUT    std_ulogic ;
      sDa    : OUT    std_ulogic 
   );
   END COMPONENT;
   COMPONENT proximitySensorSequencer
   GENERIC (
      dataBitNb         : positive := 8;
      sensorNb          : positive := 2;
      ambientLightBitNb : positive := 16;
      proximityBitNb    : positive := 16
   );
   PORT (
      clock        : IN     std_ulogic ;
      reset        : IN     std_ulogic ;
      ambientLight : OUT    unsigned (sensorNb*ambientLightBitNb-1 DOWNTO 0);
      proximity    : OUT    unsigned (sensorNb*proximityBitNb-1 DOWNTO 0);
      start        : IN     std_ulogic ;
      txData       : OUT    std_ulogic_vector (dataBitNb-1 DOWNTO 0);
      txBusy       : IN     std_ulogic ;
      txSend       : OUT    std_ulogic ;
      rxDataValid  : IN     std_ulogic ;
      rxData       : IN     std_ulogic_vector (sensorNb*dataBitNb-1 DOWNTO 0)
   );
   END COMPONENT;

   -- Optional embedded configurations
   -- pragma synthesis_off
--    FOR ALL : i2cTransmitter USE ENTITY I2C.i2cTransmitter;
--    FOR ALL : proximitySensorSequencer USE ENTITY Sensors.proximitySensorSequencer;
   -- pragma synthesis_on


BEGIN
   -- Architecture concurrent statements
   -- HDL Embedded Text Block 1 eb1
   sClOut <= (others => sClOut1);
   sDaOut <= (others => sDaOut1);        


   -- Instance port mappings.
   I_tx : i2cTransmitter
      GENERIC MAP (
         dataBitNb      => dataBitNb,
         baudRateDivide => baudRateDivide
      )
      PORT MAP (
         sCl    => sClOut1,
         clock  => clock,
         reset  => reset,
         dataIn => txData,
         send   => txSend,
         busy   => txBusy,
         sDa    => sDaOut1
      );
   I_seq : proximitySensorSequencer
      GENERIC MAP (
         dataBitNb         => dataBitNb,
         sensorNb          => sensorNb,
         ambientLightBitNb => ambientLightBitNb,
         proximityBitNb    => proximityBitNb
      )
      PORT MAP (
         clock        => clock,
         reset        => reset,
         ambientLight => ambientLight,
         proximity    => proximity,
         start        => start,
         txData       => txData,
         txBusy       => txBusy,
         txSend       => txSend,
         rxDataValid  => dataValid(1),
         rxData       => rxData
      );

   g0: FOR i IN 1 TO sensorNb GENERATE
   -- Optional embedded configurations
   -- pragma synthesis_off
--    FOR ALL : i2cReceiver USE ENTITY I2C.i2cReceiver;
   -- pragma synthesis_on

   BEGIN
      I_rx : i2cReceiver
         GENERIC MAP (
            dataBitNb => dataBitNb
         )
         PORT MAP (
            sDa => sDaIn(i),
            clock => clock,
            reset => reset,
            sCl => sClIn(i),
            dataOut => rxData(dataBitNb-1+dataBitNb*(i-1) downto dataBitNb*(i-1)),
            dataValid => dataValid(i)
         );
   END GENERATE g0;

END struct;




-- VHDL Entity Sensors.ultrasoundRanger.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:35 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY ultrasoundRanger IS
   GENERIC( 
      rangeBitNb   : positive := 16;
      counterBitNb : positive := 4
   );
   PORT( 
      clock         : IN     std_ulogic;
      distancePulse : IN     std_ulogic;
      reset         : IN     std_ulogic;
      distance      : OUT    unsigned (rangeBitNb-1 DOWNTO 0);
      startMeasure  : OUT    std_ulogic
   );

-- Declarations

END ultrasoundRanger ;





ARCHITECTURE singleCounter OF ultrasoundRanger IS

  signal timeCounter: unsigned(counterBitNb-1 downto 0);
  signal pulseDelayed, pulseEnd: std_ulogic;

BEGIN
  ------------------------------------------------------------------------------
                                                  -- generate a measurement ramp
  periodCounter: process(reset, clock)
  begin
    if reset = '1' then
      timeCounter <= (others => '0');
    elsif rising_edge(clock) then
      timeCounter <= timeCounter + 1;
    end if;
  end process periodCounter;

  startmeasure <= '1' when timeCounter < 100
    else '0';

  ------------------------------------------------------------------------------
                                                            -- find end of pulse
  delayPulse: process(reset, clock)
  begin
    if reset = '1' then
      pulseDelayed <= '0';
    elsif rising_edge(clock) then
      pulseDelayed <= distancePulse;
    end if;
  end process delayPulse;

  pulseEnd <= '1' when (distancePulse = '0') and (pulseDelayed = '1')
    else '0';

  ------------------------------------------------------------------------------
                                                               -- store distance
  storeDistance: process(reset, clock)
  begin
    if reset = '1' then
      distance <= (others => '0');
    elsif rising_edge(clock) then
      if pulseEnd = '1' then
        distance <= resize(timeCounter, distance'length);
      end if;
    end if;
  end process storeDistance;

END ARCHITECTURE singleCounter;




-- VHDL Entity Sensors.sensorsRegisters.symbol
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:35 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY sensorsRegisters IS
   GENERIC( 
      busAddressBitNb      : positive := 8;
      busDataBitNb         : positive := 8;
      ledsBaseAddress      : natural  := 12;
      ledNb                : positive := 4;
      endSwitchBaseAddress : positive := 8;
      endSwitchNb          : positive := 1;
      hallBaseAddress      : natural  := 0;
      hallSensorNb         : positive := 2;
      hallCountBitNb       : positive := 16;
      proximityBaseAddress : natural  := 12;
      proximitySensorNb    : positive := 2;
      proximityBitNb       : positive := 16;
      ambientLightBitNb    : positive := 16;
      rangeBaseAddress     : natural  := 12;
      rangeBitNb           : positive := 16
   );
   PORT( 
      clock            : IN     std_ulogic;
      reset            : IN     std_ulogic;
      busDataIn        : IN     std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      en               : IN     std_ulogic;
      busAddress       : IN     unsigned (busAddressBitNb-1 DOWNTO 0);
      leds             : OUT    std_ulogic_vector (1 TO ledNb);
      write            : IN     std_ulogic;
      busDataOut       : OUT    std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      hallCount        : IN     unsigned (hallSensorNb*hallCountBitNb-1 DOWNTO 0);
      zeroHallCounters : OUT    std_ulogic;
      proxyStart       : OUT    std_ulogic;
      proxyLight       : IN     unsigned (proximitySensorNb*ambientLightBitNb-1 DOWNTO 0);
      proxyDistance    : IN     unsigned (proximitySensorNb*proximityBitNb-1 DOWNTO 0);
      endSwitches      : IN     std_ulogic_vector (endSwitchNb-1 DOWNTO 0);
      rangerDistance   : IN     unsigned (rangeBitNb-1 DOWNTO 0)
   );

-- Declarations

END sensorsRegisters ;





ARCHITECTURE RTL OF sensorsRegisters IS

  constant registerLength: positive := 2*busDataIn'length;
  signal enDelayed: std_ulogic;

  signal leds_int: unsigned(registerLength-1 downto 0);

  type counterSetType is array(1 to hallSensorNb) of unsigned(hallCountBitNb-1 downto 0);
  signal counterSet: counterSetType;

  constant lightBaseAddress: natural := proximityBaseAddress + 2*proximitySensorNb;
  type lightSetType is array(1 to proximitySensorNb) of unsigned(ambientLightBitNb-1 downto 0);
  signal lightSet: lightSetType;
  type proximitySetType is array(1 to proximitySensorNb) of unsigned(proximityBitNb-1 downto 0);
  signal proximitySet: proximitySetType;

BEGIN
  ------------------------------------------------------------------------------
                                                         -- store bytes from I2C
  storeBytes: process(reset, clock)
  begin
    if reset = '1' then
      leds_int <= (others => '0');
    elsif rising_edge(clock) then
      if en = '1' then
        if write = '1' then
          case to_integer(busAddress) is
            when ledsBaseAddress =>
              leds_int(busDataIn'range) <= unsigned(busDataIn);
            when ledsBaseAddress+1 =>
              leds_int(
                leds_int'high downto leds_int'length-busDataIn'length
              ) <= unsigned(busDataIn);
            when others => null;
          end case;
        end if;
      end if;
    end if;
  end process storeBytes;

  proxyStart <= '1' when (en = '1') and (write = '0') and (busAddress = lightBaseAddress + 2*lightSet'length - 1)
    else '0';

  ------------------------------------------------------------------------------
                                                                 -- update words
  storeValues: process(reset, clock)
  begin
    if reset = '1' then
      enDelayed <= '0';
      leds <= (others => '0');
    elsif rising_edge(clock) then
      enDelayed <= en;
      if enDelayed = '1' then
        if busAddress(0) = '1' then
          for index in leds'range loop
            leds(index) <= leds_int(index-1);
          end loop;
        end if;
      end if;
    end if;
  end process storeValues;

  zeroHallCounters <= '0';

  --============================================================================
                                               -- build a vector of hall counters
  orderHallCountData: process(hallCount)
    variable offset : natural;
  begin
    for index in counterSet'range loop
      offset := (index-1) * counterSet(index)'length;
      counterSet(index) <= hallCount(counterSet(index)'high+offset downto offset);
    end loop;
  end process orderHallCountData;
                                         -- build a vector of ambient light data
  orderAmbientLightData: process(proxyLight)
    variable offset : natural;
  begin
    for index in lightSet'range loop
      offset := (index-1) * lightSet(index)'length;
      lightSet(index) <= proxyLight(lightSet(index)'high+offset downto offset);
    end loop;
  end process orderAmbientLightData;
                                             -- build a vector of proximity data
  orderProximityData: process(proxyDistance)
    variable offset : natural;
  begin
    for index in proximitySet'range loop
      offset := (index-1) * proximitySet(index)'length;
      proximitySet(index) <= proxyDistance(proximitySet(index)'high+offset downto offset);
    end loop;
  end process orderProximityData;

  --============================================================================
                                                        -- multiplex data to I2C
  sendStatus: process(
    busAddress,
    endSwitches, rangerDistance,
    counterSet,
    proximitySet, lightSet
  )
    variable hallIndex: natural;
    variable proximityIndex: natural;
    variable lightIndex: natural;
  begin
    busdataOut <= (others => '1');
    case to_integer(busAddress) is
                                                                 -- end switches
      when endSwitchBaseAddress =>
        busdataOut(endSwitches'range) <= not endSwitches;
                                                              -- distance ranger
      when rangeBaseAddress =>
        busdataOut <= std_ulogic_vector(rangerDistance(busdataOut'range));
      when rangeBaseAddress+1 =>
        busdataOut <= std_ulogic_vector(
          shift_right(rangerDistance, busdataOut'length)(busdataOut'range)
        );
                                                                 -- hall sensors
      when hallBaseAddress to hallBaseAddress + 2*counterSet'length - 1 =>
        hallIndex := (to_integer(busAddress) - hallBaseAddress) / 2 + 1;
        if busAddress(0) = '0' then
          busdataOut <= std_ulogic_vector(
            resize(counterSet(hallIndex), busdataOut'length)
          );
        else
          busdataOut <= std_ulogic_vector(
            resize(shift_right(counterSet(hallIndex), busdataOut'length), busdataOut'length)
          );
        end if;
                                                                    -- proximity
      when proximityBaseAddress to proximityBaseAddress + 2*proximitySet'length - 1 =>
        proximityIndex := (to_integer(busAddress) - proximityBaseAddress) / 2 + 1;
        if busAddress(0) = '0' then
          busdataOut <= std_ulogic_vector(
            resize(proximitySet(proximityIndex), busdataOut'length)
          );
        else
          busdataOut <= std_ulogic_vector(
            resize(shift_right(proximitySet(proximityIndex), busdataOut'length), busdataOut'length)
          );
        end if;
                                                                        -- light
      when lightBaseAddress to lightBaseAddress + 2*lightSet'length - 1 =>
        lightIndex := (to_integer(busAddress) - lightBaseAddress) / 2 + 1;
        if busAddress(0) = '0' then
          busdataOut <= std_ulogic_vector(
            resize(lightSet(lightIndex), busdataOut'length)
          );
        else
          busdataOut <= std_ulogic_vector(
            resize(shift_right(lightSet(lightIndex), busdataOut'length), busdataOut'length)
          );
        end if;
      when others => null;
    end case;
  end process sendStatus;

END ARCHITECTURE RTL;




--
-- VHDL Architecture Sensors.sensorsInterface.struct
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:35 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

-- LIBRARY I2C;
-- LIBRARY Sensors;

ARCHITECTURE struct OF sensorsInterface IS

   -- Architecture declarations
   constant dataBitNb: positive := i2cBitNb-2;
   constant addressBitNb: positive := dataBitNb;

   -- Internal signal declarations
   SIGNAL addr             : unsigned(addressBitNb-1 DOWNTO 0);
   SIGNAL chipAddr         : unsigned(dataBitNb-1-1 DOWNTO 0);
   SIGNAL dataIn           : std_ulogic_vector(dataBitNb-1 DOWNTO 0);
   SIGNAL dataOut          : std_ulogic_vector(dataBitNb-1 DOWNTO 0);
   SIGNAL dataValid        : std_ulogic;
   SIGNAL hallChanged      : std_ulogic_vector(1 TO hallSensorNb);
   SIGNAL hallCount        : unsigned(hallSensorNb*hallCountBitNb-1 DOWNTO 0);
   SIGNAL isSelected       : std_ulogic;
   SIGNAL proxyDistance    : unsigned(proximitySensorNb*proximityBitNb-1 DOWNTO 0);
   SIGNAL proxyLight       : unsigned(proximitySensorNb*ambientLightBitNb-1 DOWNTO 0);
   SIGNAL proxyStart       : std_ulogic;
   SIGNAL rangerDistance   : unsigned(rangeBitNb-1 DOWNTO 0);
   SIGNAL writeData        : std_ulogic;
   SIGNAL zeroHallCounters : std_ulogic;


   -- Component Declarations
   COMPONENT i2cTransceiver
   GENERIC (
      dataBitNb : positive := 10
   );
   PORT (
      sDaIn        : IN     std_ulogic ;
      clock        : IN     std_ulogic ;
      reset        : IN     std_ulogic ;
      dataIn       : OUT    std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      dataValid    : OUT    std_ulogic ;
      sCl          : IN     std_ulogic ;
      registerAddr : OUT    unsigned (dataBitNb-2-1 DOWNTO 0);
      writeData    : OUT    std_ulogic ;
      sDaOut       : OUT    std_ulogic ;
      dataOut      : IN     std_ulogic_vector (dataBitNb-2-1 DOWNTO 0);
      chipAddr     : OUT    unsigned (dataBitNb-3-1 DOWNTO 0);
      isSelected   : IN     std_ulogic 
   );
   END COMPONENT;
   COMPONENT hallCounters
   GENERIC (
      sensorNb   : positive := 2;
      countBitNb : positive := 16
   );
   PORT (
      clock       : IN     std_ulogic ;
      hallPulses  : IN     std_ulogic_vector (1 TO sensorNb);
      reset       : IN     std_ulogic ;
      zeroPos     : IN     std_ulogic ;
      hallChanged : OUT    std_ulogic_vector (1 TO sensorNb);
      position    : OUT    unsigned (sensorNb*countBitNb-1 DOWNTO 0)
   );
   END COMPONENT;
   COMPONENT proximitySensorInterface
   GENERIC (
      dataBitNb         : positive := 10;
      baudRateDivide    : positive := 660;
      sensorNb          : positive := 4;
      ambientLightBitNb : positive := 16;
      proximityBitNb    : positive := 16
   );
   PORT (
      clock        : IN     std_ulogic ;
      reset        : IN     std_ulogic ;
      ambientLight : OUT    unsigned (sensorNb*ambientLightBitNb-1 DOWNTO 0);
      sDaOut       : OUT    std_ulogic_vector (1 TO sensorNb);
      sClOut       : OUT    std_ulogic_vector (1 TO sensorNb);
      sClIn        : IN     std_ulogic_vector (1 TO sensorNb);
      sDaIn        : IN     std_ulogic_vector (1 TO sensorNb);
      start        : IN     std_ulogic ;
      proximity    : OUT    unsigned (sensorNb*proximityBitNB-1 DOWNTO 0)
   );
   END COMPONENT;
   COMPONENT sensorsRegisters
   GENERIC (
      busAddressBitNb      : positive := 8;
      busDataBitNb         : positive := 8;
      ledsBaseAddress      : natural  := 12;
      ledNb                : positive := 4;
      endSwitchBaseAddress : positive := 8;
      endSwitchNb          : positive := 1;
      hallBaseAddress      : natural  := 0;
      hallSensorNb         : positive := 2;
      hallCountBitNb       : positive := 16;
      proximityBaseAddress : natural  := 12;
      proximitySensorNb    : positive := 2;
      proximityBitNb       : positive := 16;
      ambientLightBitNb    : positive := 16;
      rangeBaseAddress     : natural  := 12;
      rangeBitNb           : positive := 16
   );
   PORT (
      clock            : IN     std_ulogic ;
      reset            : IN     std_ulogic ;
      busDataIn        : IN     std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      en               : IN     std_ulogic ;
      busAddress       : IN     unsigned (busAddressBitNb-1 DOWNTO 0);
      leds             : OUT    std_ulogic_vector (1 TO ledNb);
      write            : IN     std_ulogic ;
      busDataOut       : OUT    std_ulogic_vector (busDataBitNb-1 DOWNTO 0);
      hallCount        : IN     unsigned (hallSensorNb*hallCountBitNb-1 DOWNTO 0);
      zeroHallCounters : OUT    std_ulogic ;
      proxyStart       : OUT    std_ulogic ;
      proxyLight       : IN     unsigned (proximitySensorNb*ambientLightBitNb-1 DOWNTO 0);
      proxyDistance    : IN     unsigned (proximitySensorNb*proximityBitNb-1 DOWNTO 0);
      endSwitches      : IN     std_ulogic_vector (endSwitchNb-1 DOWNTO 0);
      rangerDistance   : IN     unsigned (rangeBitNb-1 DOWNTO 0)
   );
   END COMPONENT;
   COMPONENT ultrasoundRanger
   GENERIC (
      rangeBitNb   : positive := 16;
      counterBitNb : positive := 4
   );
   PORT (
      clock         : IN     std_ulogic ;
      distancePulse : IN     std_ulogic ;
      reset         : IN     std_ulogic ;
      distance      : OUT    unsigned (rangeBitNb-1 DOWNTO 0);
      startMeasure  : OUT    std_ulogic 
   );
   END COMPONENT;

   -- Optional embedded configurations
   -- pragma synthesis_off
--    FOR ALL : hallCounters USE ENTITY Sensors.hallCounters;
--    FOR ALL : i2cTransceiver USE ENTITY I2C.i2cTransceiver;
--    FOR ALL : proximitySensorInterface USE ENTITY Sensors.proximitySensorInterface;
--    FOR ALL : sensorsRegisters USE ENTITY Sensors.sensorsRegisters;
--    FOR ALL : ultrasoundRanger USE ENTITY Sensors.ultrasoundRanger;
   -- pragma synthesis_on


BEGIN
   -- Architecture concurrent statements
   -- HDL Embedded Text Block 1 eb1
   testOut <= (
     1 => proxyStart,
     2 => proxyDistance(8),
     3 => proxyDistance(7),
     4 => hallChanged(1),
     5 => hallChanged(2),
     others => '0'
   );

   -- HDL Embedded Text Block 2 eb2
   isSelected <='1' when chipAddr = kartBaseAddress/2 else '0';
   
   


   -- Instance port mappings.
   I_i2c : i2cTransceiver
      GENERIC MAP (
         dataBitNb => i2cBitNb
      )
      PORT MAP (
         sDaIn        => sDaIn,
         clock        => clock,
         reset        => reset,
         dataIn       => dataIn,
         dataValid    => dataValid,
         sCl          => sClIn,
         registerAddr => addr,
         writeData    => writeData,
         sDaOut       => sDaOut,
         dataOut      => dataOut,
         chipAddr     => chipAddr,
         isSelected   => isSelected
      );
   I_hall : hallCounters
      GENERIC MAP (
         sensorNb   => hallSensorNb,
         countBitNb => hallCountBitNb
      )
      PORT MAP (
         clock       => clock,
         hallPulses  => hallPulses,
         reset       => reset,
         zeroPos     => zeroHallCounters,
         hallChanged => hallChanged,
         position    => hallCount
      );
   I_prox : proximitySensorInterface
      GENERIC MAP (
         dataBitNb         => i2cBitNb,
         baudRateDivide    => proximityBaudRateDivide,
         sensorNb          => proximitySensorNb,
         ambientLightBitNb => ambientLightBitNb,
         proximityBitNb    => proximityBitNb
      )
      PORT MAP (
         clock        => clock,
         reset        => reset,
         ambientLight => proxyLight,
         sDaOut       => proxySDaOut,
         sClOut       => proxySClOut,
         sClIn        => proxySClIn,
         sDaIn        => proxySDaIn,
         start        => proxyStart,
         proximity    => proxyDistance
      );
   I_regs : sensorsRegisters
      GENERIC MAP (
         busAddressBitNb      => addressBitNb,
         busDataBitNb         => dataBitNb,
         ledsBaseAddress      => ledsBaseAddress,
         ledNb                => ledNb,
         endSwitchBaseAddress => endSwitchBaseAddress,
         endSwitchNb          => endSwitchNb,
         hallBaseAddress      => hallBaseAddress,
         hallSensorNb         => hallSensorNb,
         hallCountBitNb       => hallCountBitNb,
         proximityBaseAddress => proximityBaseAddress,
         proximitySensorNb    => proximitySensorNb,
         proximityBitNb       => proximityBitNb,
         ambientLightBitNb    => ambientLightBitNb,
         rangeBaseAddress     => rangeBaseAddress,
         rangeBitNb           => rangeBitNb
      )
      PORT MAP (
         clock            => clock,
         reset            => reset,
         busDataIn        => dataIn,
         en               => dataValid,
         busAddress       => addr,
         leds             => leds,
         write            => writeData,
         busDataOut       => dataOut,
         hallCount        => hallCount,
         zeroHallCounters => zeroHallCounters,
         proxyStart       => proxyStart,
         proxyLight       => proxyLight,
         proxyDistance    => proxyDistance,
         endSwitches      => endSwitches,
         rangerDistance   => rangerDistance
      );
   I_range : ultrasoundRanger
      GENERIC MAP (
         rangeBitNb   => rangeBitNb,
         counterBitNb => rangeCounterBitNb
      )
      PORT MAP (
         clock         => clock,
         distancePulse => distancePulse,
         reset         => reset,
         distance      => rangerDistance,
         startMeasure  => distanceStart
      );

END struct;




LIBRARY ieee;
USE ieee.std_logic_1164.all;

PACKAGE gates IS

--  constant gateDelay: time := 1 ns;
  constant gateDelay: time := 0.1 ns;

END gates;




-- VHDL Entity sequential.DFF.symbol
--
-- Created:
--          by - cof.UNKNOWN (WE3479)
--          at - 10:57:17 23.08.2012
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
-- LIBRARY gates;
  use work.gates.all;

ENTITY DFF IS
  GENERIC( 
    delay : time := 1 ns
  );
  PORT( 
    CLK : IN     std_uLogic;
    CLR : IN     std_uLogic;
    D   : IN     std_uLogic;
    Q   : OUT    std_uLogic
  );

-- Declarations

END DFF ;





ARCHITECTURE sim OF DFF IS
BEGIN

  process(clk, clr)
  begin
    if clr = '1' then
      q <= '0' after delay;
    elsif rising_edge(clk) then
      q <= d after delay;
    end if;
  end process;

END sim;




-- VHDL Entity gates.inverter.symbol
--
-- Created:
--          by - cof.UNKNOWN (WE3479)
--          at - 10:55:55 23.08.2012
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
-- LIBRARY gates;
  use work.gates.all;

ENTITY inverter IS
  GENERIC( 
    delay : time := 1 ns
  );
  PORT( 
    in1  : IN     std_uLogic;
    out1 : OUT    std_uLogic
  );

-- Declarations

END inverter ;





ARCHITECTURE sim OF inverter IS
BEGIN
  out1 <= NOT in1 after delay;
END sim;




-- VHDL Entity gates.logic1.symbol
--
-- Created:
--          by - cof.UNKNOWN (WE3479)
--          at - 10:55:54 23.08.2012
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
-- LIBRARY gates;
  use work.gates.all;

ENTITY logic1 IS
  PORT( 
    logic_1 : OUT    std_uLogic
  );

-- Declarations

END logic1 ;





ARCHITECTURE sim OF logic1 IS
BEGIN
  logic_1 <= '1';
END sim;




--
-- VHDL Architecture Board.FPGA_sensorsInterface.struct
--
-- Created:
--          by - guo.UNKNOWN (WEA20309)
--          at - 10:09:36 28.08.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.numeric_std.all;

-- LIBRARY gates;
   use work.gates.all;

-- LIBRARY gates;
-- LIBRARY Sensors;
-- LIBRARY sequential;

ARCHITECTURE struct OF FPGA_sensorsInterface IS

   -- Architecture declarations
   constant clockFrequency: real := 10.0E6;
   constant i2cBitNb: positive := 10;
   
   constant kartBaseAddress: positive := 16#26#; -- must be even
   constant ledsBaseAddress: natural := 12;
   constant ledNb: positive := 4;
   constant endSwitchBaseAddress: natural := 8;
   constant endSwitchNb: positive := 1;
   constant hallBaseAddress: natural := 0;
   constant hallSensorNb: positive := 2;
   constant hallCountBitNb: positive := 16;
   constant proximityBaseAddress: natural := 16;
   constant proximityBaudRate: real := 100.0E3;
   constant proximityBaudRateDivide: positive := integer(clockFrequency/proximityBaudRate / 4.0);
   constant proximitySensorNb: positive := 4;
   constant ambientLightBitNb: positive := 16;
   constant proximityBitNb: positive := 16;
   constant rangeBaseAddress: natural := 12;
   constant rangeBitNb: positive := 16;
   constant rangeCounterBitNb: positive := 16;
   
   constant testOutBitNb : positive := 8;

   -- Internal signal declarations
   SIGNAL I_O_synch     : std_ulogic_vector(1 TO ioBitNb);
   SIGNAL Q             : std_uLogic_vector(1 TO 1);
   SIGNAL distancePulse : std_ulogic;
   SIGNAL distanceStart : std_ulogic;
   SIGNAL endSwitches   : std_ulogic_vector(1 TO endSwitchNb);
   SIGNAL hallPulses    : std_ulogic_vector(1 TO hallSensorNb);
   SIGNAL leds          : std_ulogic_vector(1 TO ledNb);
   SIGNAL logic_1       : std_uLogic;
   SIGNAL proxySClIn    : std_ulogic_vector(1 TO proximitySensorNb);
   SIGNAL proxySClOut   : std_ulogic_vector(1 TO proximitySensorNb);
   SIGNAL proxySDaIn    : std_ulogic_vector(1 TO proximitySensorNb);
   SIGNAL proxySDaOut   : std_ulogic_vector(1 TO proximitySensorNb);
   SIGNAL reset         : std_uLogic;
   SIGNAL resetSynch    : std_ulogic;
   SIGNAL resetSynch_n  : std_ulogic;
   SIGNAL sClSynch      : std_uLogic;
   SIGNAL sDaIn         : std_uLogic;
   SIGNAL sDaInSynch    : std_uLogic;
   SIGNAL sDaOut        : std_uLogic;
   SIGNAL testOut       : std_ulogic_vector(1 TO testOutBitNb);


   -- Component Declarations
   COMPONENT inverter
   GENERIC (
      delay : time := 1 ns
   );
   PORT (
      in1  : IN     std_uLogic ;
      out1 : OUT    std_uLogic 
   );
   END COMPONENT;
   COMPONENT logic1
   PORT (
      logic_1 : OUT    std_uLogic 
   );
   END COMPONENT;
   COMPONENT sensorsInterface
   GENERIC (
      i2cBitNb                : positive := 10;
      kartBaseAddress         : positive := 16#51#;
      ledsBaseAddress         : natural  := 12;
      ledNb                   : positive := 4;
      testOutBitNb            : positive := 8;
      endSwitchBaseAddress    : natural  := 8;
      endSwitchNb             : positive := 1;
      hallBaseAddress         : natural  := 0;
      hallSensorNb            : positive := 2;
      hallCountBitNb          : positive := 16;
      proximityBaudRateDivide : positive := 100;
      proximityBaseAddress    : natural  := 12;
      proximitySensorNb       : positive := 2;
      ambientlightBitNb       : positive := 16;
      proximityBitNb          : positive := 16;
      rangeBaseAddress        : natural  := 16;
      rangeBitNb              : positive := 16;
      rangeCounterBitNb       : positive := 16
   );
   PORT (
      clock         : IN     std_ulogic ;
      distancePulse : IN     std_ulogic ;
      endSwitches   : IN     std_ulogic_vector (1 TO endSwitchNb);
      hallPulses    : IN     std_ulogic_vector (1 TO hallSensorNb);
      proxySClIn    : IN     std_ulogic_vector (1 TO proximitySensorNb);
      proxySDaIn    : IN     std_ulogic_vector (1 TO proximitySensorNb);
      reset         : IN     std_ulogic ;
      sClIn         : IN     std_uLogic ;
      sDaIn         : IN     std_uLogic ;
      distanceStart : OUT    std_ulogic ;
      leds          : OUT    std_ulogic_vector (1 TO ledNb);
      proxySClOut   : OUT    std_ulogic_vector (1 TO proximitySensorNb);
      proxySDaOut   : OUT    std_ulogic_vector (1 TO proximitySensorNb);
      sDaOut        : OUT    std_uLogic ;
      testOut       : OUT    std_ulogic_vector (1 TO testOutBitNb)
   );
   END COMPONENT;
   COMPONENT DFF
   GENERIC (
      delay : time := 1 ns
   );
   PORT (
      CLK : IN     std_uLogic ;
      CLR : IN     std_uLogic ;
      D   : IN     std_uLogic ;
      Q   : OUT    std_uLogic 
   );
   END COMPONENT;

   -- Optional embedded configurations
   -- pragma synthesis_off
--    FOR ALL : DFF USE ENTITY sequential.DFF;
--    FOR ALL : inverter USE ENTITY gates.inverter;
--    FOR ALL : logic1 USE ENTITY gates.logic1;
--    FOR ALL : sensorsInterface USE ENTITY Sensors.sensorsInterface;
   -- pragma synthesis_on


BEGIN
   -- Architecture concurrent statements
   -- HDL Embedded Text Block 2 eb2
   jtagIo(1) <= '0';
   jtagIo(2) <= '0';
   jtagIo(3) <= '0';
   

   -- HDL Embedded Text Block 3 eb3
   sDa <= '0' when sDaOut = '0'
     else 'Z';
   sDaIn <= sDa;
   

   -- HDL Embedded Text Block 4 eb4
   I_O(1)  <= 'Z';         -- distance sensor pulse
   I_O(2)  <= 'Z';         -- end switch
   I_O(3)  <= 'Z';         -- Hall pulses 1
   I_O(4)  <= 'Z';         -- Hall pulses 2
   I_O(5)  <= 'Z';         -- proximity SCL
   I_O(6)  <= 'Z';         -- proximity SDA 1
   I_O(7)  <= 'Z';         -- proximity SDA 2
   I_O(8)  <= 'Z';         -- proximity SDA 3
   I_O(9)  <= 'Z';         -- proximity SDA 4
   I_O(10) <= testOut(4);
   I_O(11) <= testOut(5);
   I_O(12) <= Q(1);        -- trigger for proximity sensor I2C
   I_O(13) <= '0';
   I_O(14) <= '0';
   I_O(15) <= '0';
   I_O(16) <= 'Z';         -- distance sensor start
   I_O(17) <= 'Z';         -- LED 1
   I_O(18) <= 'Z';         -- LED 2
   I_O(19) <= 'Z';         -- LED 3
   I_O(20) <= 'Z';         -- LED 4
   
   
   div2: for index in Q'range generate
   begin
     process(resetSynch, clock)
     begin
       if resetSynch = '1' then
         Q(index) <= '0';
       elsif rising_edge(clock) then
           if testOut(index) = '1' then
             Q(index) <= not Q(index);
           end if;
       end if;
     end process;
   end generate div2;
   
   
   
   
   

   -- HDL Embedded Text Block 5 eb5
   I_O(17 to 20) <= std_logic_vector(leds);
   
   endSwitches <=  I_O_synch(2 to 2);
   
   hallPulses <= std_ulogic_vector(I_O(3 to 4));
   
   I_O(16) <= distanceStart;
   distancePulse <= I_O_synch(1);
   
   I_O(5) <= proxySClOut(1);
   driveI2cSDa: for index in 1 to 4 generate
     I_O(5+index) <= '0' when proxySDaOut(index) = '0' else 'Z';
     proxySClIn(index) <= I_O_synch(5);
     proxySDaIn(index) <= I_O_synch(5+index);
   end generate driveI2cSDa;
   
   synchInputs: process(resetSynch, clock)
   begin
     if resetSynch = '1' then
       I_O_synch <= (others => '0');
     elsif rising_edge(clock) then
       I_O_synch <= std_ulogic_vector(I_O);
     end if;
   end process synchInputs;
   


   -- Instance port mappings.
   I26 : inverter
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         in1  => resetSynch_n,
         out1 => resetSynch
      );
   I27 : inverter
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         in1  => reset_n,
         out1 => reset
      );
   I3 : logic1
      PORT MAP (
         logic_1 => logic_1
      );
   I1 : sensorsInterface
      GENERIC MAP (
         i2cBitNb                => i2cBitNb,
         kartBaseAddress         => kartBaseAddress,
         ledsBaseAddress         => ledsBaseAddress,
         ledNb                   => ledNb,
         testOutBitNb            => testOutBitNb,
         endSwitchBaseAddress    => endSwitchBaseAddress,
         endSwitchNb             => endSwitchNb,
         hallBaseAddress         => hallBaseAddress,
         hallSensorNb            => hallSensorNb,
         hallCountBitNb          => hallCountBitNb,
         proximityBaudRateDivide => proximitybaudRateDivide,
         proximityBaseAddress    => proximityBaseAddress,
         proximitySensorNb       => proximitySensorNb,
         ambientlightBitNb       => ambientLightBitNb,
         proximityBitNb          => proximityBitNb,
         rangeBaseAddress        => rangeBaseAddress,
         rangeBitNb              => rangeBitNb,
         rangeCounterBitNb       => rangeCounterBitNb
      )
      PORT MAP (
         clock         => clock,
         distancePulse => distancePulse,
         endSwitches   => endSwitches,
         hallPulses    => hallPulses,
         proxySClIn    => proxySClIn,
         proxySDaIn    => proxySDaIn,
         reset         => resetSynch,
         sClIn         => sClSynch,
         sDaIn         => sDaInSynch,
         distanceStart => distanceStart,
         leds          => leds,
         proxySClOut   => proxySClOut,
         proxySDaOut   => proxySDaOut,
         sDaOut        => sDaOut,
         testOut       => testOut
      );
   I21 : DFF
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         CLK => clock,
         CLR => reset,
         D   => logic_1,
         Q   => resetSynch_n
      );
   I23 : DFF
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         CLK => clock,
         CLR => resetSynch,
         D   => sCl,
         Q   => sClSynch
      );
   I24 : DFF
      GENERIC MAP (
         delay => 1 ns
      )
      PORT MAP (
         CLK => clock,
         CLR => resetSynch,
         D   => sDaIn,
         Q   => sDaInSynch
      );

END struct;




